class FormChecker {
  int trigPin;
  int echoPin;
  int maxDistance;
  unsigned long sampleInterval;
  unsigned long lastSampleTime;
  unsigned long pulseStartTime;
  unsigned long pulseEndTime;
  int lastDistance;
  int state; // 0 = Idle, 1 = Triggering, 2 = Waiting for Echo

  // Rolling average variables
  static const int rollingDuration = 60000; // 1 minute in milliseconds
  unsigned long rollingStartTime;
  int rollingSum;
  int rollingCount;

public:
  FormChecker(int trig, int echo, int maxDist, unsigned long sampleInt = 50) {
    trigPin = trig;
    echoPin = echo;
    maxDistance = maxDist;
    sampleInterval = sampleInt;
    lastSampleTime = 0;
    lastDistance = -1;
    state = 0; // Start in idle state

    rollingStartTime = millis();
    rollingSum = 0;
    rollingCount = 0;
    
    pinMode(trigPin, OUTPUT);
    pinMode(echoPin, INPUT);
  }

  void update() {
    unsigned long currentTime = millis();
    
    // Only start a new measurement if enough time has passed
    if (state == 0 && (currentTime - lastSampleTime >= sampleInterval)) {
      lastSampleTime = currentTime;
      digitalWrite(trigPin, LOW);
      pulseStartTime = micros(); // Mark start time
      state = 1; // Move to next state
    }

    if (state == 1 && (micros() - pulseStartTime >= 2)) {
      digitalWrite(trigPin, HIGH);
      pulseStartTime = micros(); // Update start time for HIGH pulse
      state = 2;
    }

    if (state == 2 && (micros() - pulseStartTime >= 5)) {
      digitalWrite(trigPin, LOW);
      state = 3;
    }

    if (state == 3) {
      pulseEndTime = pulseIn(echoPin, HIGH, maxDistance * 59); // Read echo duration
      lastDistance = pulseEndTime / 59; // Convert to cm
      state = 0; // Reset for next measurement
      
      // Add measurement to rolling average
      rollingSum += lastDistance;
      rollingCount++;
    }
  }

  int getDistance() {
    return lastDistance;
  }

  float getRollingAverage() {
    unsigned long currentTime = millis();
    if (currentTime - rollingStartTime >= rollingDuration) {
      float average = (rollingCount > 0) ? (float)rollingSum / rollingCount : 0;
      
      // Reset rolling average
      rollingStartTime = currentTime;
      rollingSum = 0;
      rollingCount = 0;

      return average;
    }
    return -1; // Indicate that 1 minute hasn't passed yet
  }

  bool isFormBad() {
    float rollingAverage = getRollingAverage();
    if (rollingAverage == 0) {
      return true;
    }
    return false;
  }
};
